@@ -143,6 +143,7 @@ add_hide_timeout (MsdOsdWindow *window)
                                                        window);
 }
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 void
 msd_osd_window_draw_rounded_rectangle (cairo_t* cr,
                                        gdouble  aspect,
@@ -224,6 +225,7 @@ msd_osd_window_color_reverse (const GdkColor *a,
         b->green = green * 65535.0;
         b->blue = blue * 65535.0;
 }
+#endif
 
 /* This is our expose/draw-event handler when the window is in a compositing manager.
  * We draw everything by hand, using Cairo, so that we can have a nice
@@ -231,7 +233,7 @@ msd_osd_window_color_reverse (const GdkColor *a,
  */
 static void
 #if GTK_CHECK_VERSION (3, 0, 0)
-draw_when_composited (GtkWidget *widget, cairo_t *context)
+draw_when_composited (GtkWidget *widget, cairo_t *orig_cr)
 #else
 expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
 #endif
@@ -244,9 +246,13 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
         cairo_surface_t *surface;
         int              width;
         int              height;
+#if GTK_CHECK_VERSION (3, 0, 0)
+        GtkStyleContext *context;
+#else
         GtkStyle        *style;
         GdkColor         color;
         double           r, g, b;
+#endif
 
         window = MSD_OSD_WINDOW (widget);
 
@@ -254,11 +260,20 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
         context = gdk_cairo_create (gtk_widget_get_window (widget));
 #endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        context = gtk_widget_get_style_context (widget);
+        cairo_set_operator (orig_cr, CAIRO_OPERATOR_SOURCE);
+#else
         style = gtk_widget_get_style (widget);
         cairo_set_operator (context, CAIRO_OPERATOR_SOURCE);
+#endif
         gtk_window_get_size (GTK_WINDOW (widget), &width, &height);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        surface = cairo_surface_create_similar (cairo_get_target (orig_cr),
+#else
         surface = cairo_surface_create_similar (cairo_get_target (context),
+#endif
                                                 CAIRO_CONTENT_COLOR_ALPHA,
                                                 width,
                                                 height);
@@ -271,10 +286,16 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
         if (cairo_status (cr) != CAIRO_STATUS_SUCCESS) {
                 goto done;
         }
+#if !GTK_CHECK_VERSION (3, 0, 0)
         cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
         cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
         cairo_paint (cr);
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        gtk_render_background (context, cr, 0, 0, width, height);
+        gtk_render_frame (context, cr, 0, 0, width, height);
+#else
         /* draw a box */
         msd_osd_window_draw_rounded_rectangle (cr, 1.0, 0.5, 0.5, height / 10, width-1, height-1);
         msd_osd_window_color_reverse (&style->bg[GTK_STATE_NORMAL], &color);
@@ -291,6 +312,7 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
         cairo_set_source_rgba (cr, r, g, b, BG_ALPHA / 2);
         cairo_set_line_width (cr, 1);
         cairo_stroke (cr);
+#endif
 
 #if GTK_CHECK_VERSION (3, 0, 0)
         g_signal_emit (window, signals[DRAW_WHEN_COMPOSITED], 0, cr);
@@ -301,12 +323,21 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
         cairo_destroy (cr);
 
         /* Make sure we have a transparent background */
+#if GTK_CHECK_VERSION (3, 0, 0)
+        cairo_rectangle (orig_cr, 0, 0, width, height);
+        cairo_set_source_rgba (orig_cr, 0.0, 0.0, 0.0, 0.0);
+        cairo_fill (orig_cr);
+
+        cairo_set_source_surface (orig_cr, surface, 0, 0);
+        cairo_paint_with_alpha (orig_cr, window->priv->fade_out_alpha);
+#else
         cairo_rectangle (context, 0, 0, width, height);
         cairo_set_source_rgba (context, 0.0, 0.0, 0.0, 0.0);
         cairo_fill (context);
 
         cairo_set_source_surface (context, surface, 0, 0);
         cairo_paint_with_alpha (context, window->priv->fade_out_alpha);
+#endif
 
  done:
         if (surface != NULL) {
@@ -327,22 +358,21 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
 static void
 draw_when_not_composited (GtkWidget *widget, cairo_t *cr)
 {
-	int width;
-	int height;
+        GtkStyleContext *context;
+        int width;
+        int height;
 
-	width = gtk_widget_get_allocated_width (widget);
-	height = gtk_widget_get_allocated_width (widget);
+        width = gtk_widget_get_allocated_width (widget);
+        height = gtk_widget_get_allocated_width (widget);
+        context = gtk_widget_get_style_context (widget);
 
-	gtk_paint_shadow (gtk_widget_get_style (widget),
-			  cr,
-			  gtk_widget_get_state (widget),
-			  GTK_SHADOW_OUT,
-			  widget,
-			  NULL, /* NULL detail -> themes should use the MsdOsdWindow widget name, probably */
-			  0,
-			  0,
-			  width,
-			  height);
+        gtk_style_context_set_state (context, GTK_STATE_FLAG_ACTIVE);
+        gtk_render_frame (context,
+                          cr,
+                          0,
+                          0,
+                          width,
+                          height);
 }
 #else
 static void
@@ -495,7 +525,25 @@ msd_osd_window_real_realize (GtkWidget *widget)
         cairo_destroy (cr);
 #endif
 }
+#if  GTK_CHECK_VERSION (3, 0, 0)
+static void
+msd_osd_window_style_updated (GtkWidget *widget)
+{
+        GtkStyleContext *context;
+        GtkBorder padding;
 
+        GTK_WIDGET_CLASS (msd_osd_window_parent_class)->style_updated (widget);
+
+        /* We set our border width to 12 (per the MATE standard), plus the
+         * padding of the frame that we draw in our expose/draw handler.  This will
+         * make our child be 12 pixels away from the frame.
+         */
+
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_get_padding (context, GTK_STATE_FLAG_NORMAL, &padding);
+        gtk_container_set_border_width (GTK_CONTAINER (widget), 12 + MAX (padding.left, padding.top));
+}
+#else
 static void
 msd_osd_window_style_set (GtkWidget *widget,
                           GtkStyle  *previous_style)
@@ -512,40 +560,44 @@ msd_osd_window_style_set (GtkWidget *widget,
         style = gtk_widget_get_style (widget);
         gtk_container_set_border_width (GTK_CONTAINER (widget), 12 + MAX (style->xthickness, style->ythickness));
 }
-
+#endif
 #if GTK_CHECK_VERSION (3, 0, 0)
 static void
 msd_osd_window_get_preferred_width (GtkWidget *widget,
-                                    gint *minimum_width,
-                                    gint *natural_width)
+                                    gint      *minimum,
+                                    gint      *natural)
 {
-        GtkStyle *style;
+        GtkStyleContext *context;
+        GtkBorder padding;
 
-        GTK_WIDGET_CLASS (msd_osd_window_parent_class)->get_preferred_width (widget, minimum_width, natural_width);
+        GTK_WIDGET_CLASS (msd_osd_window_parent_class)->get_preferred_width (widget, minimum, natural);
 
-        /* See the comment in msd_osd_window_style_set() for why we add the thickness here */
+        /* See the comment in msd_osd_window_style_updated() for why we add the padding here */
 
-        style = gtk_widget_get_style (widget);
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_get_padding (context, GTK_STATE_FLAG_NORMAL, &padding);
 
-        *minimum_width += style->xthickness;
-        *natural_width += style->xthickness;
+        *minimum += padding.left;
+        *natural += padding.left;
 }
 
 static void
 msd_osd_window_get_preferred_height (GtkWidget *widget,
-                                     gint *minimum_height,
-                                     gint *natural_height)
+                                     gint      *minimum,
+                                     gint      *natural)
 {
-        GtkStyle *style;
+        GtkStyleContext *context;
+        GtkBorder padding;
 
-        GTK_WIDGET_CLASS (msd_osd_window_parent_class)->get_preferred_height (widget, minimum_height, natural_height);
+        GTK_WIDGET_CLASS (msd_osd_window_parent_class)->get_preferred_height (widget, minimum, natural);
 
-        /* See the comment in msd_osd_window_style_set() for why we add the thickness here */
+        /* See the comment in msd_osd_window_style_updated() for why we add the padding here */
 
-        style = gtk_widget_get_style (widget);
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_get_padding (context, GTK_STATE_FLAG_NORMAL, &padding);
 
-        *minimum_height += style->ythickness;
-        *natural_height += style->ythickness;
+        *minimum += padding.top;
+        *natural += padding.top;
 }
 #else
 static void
@@ -582,6 +634,12 @@ msd_osd_window_constructor (GType                  type,
                       "focus-on-map", FALSE,
                       NULL);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        GtkWidget *widget = GTK_WIDGET (object);
+        GtkStyleContext *style_context = gtk_widget_get_style_context (widget);
+        gtk_style_context_add_class (style_context, "osd");
+#endif
+
         return object;
 }
 
@@ -596,12 +654,13 @@ msd_osd_window_class_init (MsdOsdWindowClass *klass)
         widget_class->show = msd_osd_window_real_show;
         widget_class->hide = msd_osd_window_real_hide;
         widget_class->realize = msd_osd_window_real_realize;
-        widget_class->style_set = msd_osd_window_style_set;
 #if GTK_CHECK_VERSION (3, 0, 0)
+        widget_class->style_updated = msd_osd_window_style_updated;
         widget_class->get_preferred_width = msd_osd_window_get_preferred_width;
         widget_class->get_preferred_height = msd_osd_window_get_preferred_height;
         widget_class->draw = msd_osd_window_draw;
 #else
+        widget_class->style_set = msd_osd_window_style_set;
         widget_class->size_request = msd_osd_window_size_request;
         widget_class->expose_event = msd_osd_window_expose_event;
 #endif
@@ -623,6 +682,10 @@ msd_osd_window_class_init (MsdOsdWindowClass *klass)
                                                         G_TYPE_NONE, 1,
                                                         G_TYPE_POINTER);
 
+#if GTK_CHECK_VERSION (3, 20, 0)
+        gtk_widget_class_set_css_name (widget_class, "MsdOsdWindow");
+#endif
+
         g_type_class_add_private (klass, sizeof (MsdOsdWindowPrivate));
 }
 
@@ -670,6 +733,11 @@ msd_osd_window_init (MsdOsdWindow *window)
                 gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
                 gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+                GtkStyleContext *style = gtk_widget_get_style_context (GTK_WIDGET (window));
+                gtk_style_context_add_class (style, "window-frame");
+#endif
+
                 /* assume 130x130 on a 640x480 display and scale from there */
                 scalew = gdk_screen_get_width (screen) / 640.0;
                 scaleh = gdk_screen_get_height (screen) / 480.0;
